###################################################################
# VacationAdm Webmin module                                       #
###################################################################
#                                                                 #
# $Id: local-lib.pl,v 1.1.1.1 2003/02/27 22:18:24 martin Exp $
#                                                                 #
###################################################################

# local-lib.pl
# Generic function for module
# Generique pour tous les modules

do '../web-lib.pl';
&init_config();


#
# Here follow some helper functions that we need
#

# Pretty much all pages begin this way
sub html_open {
&header($text{'lib_header_results'}, "");  
}


# All pages are closed this way
sub html_close {
    &footer("index.cgi", "Vacation Admin");
    unlink($tmp);
    exit;
}


# Oops perverse situation -- die cleanly closing off the html
sub html_abort {
    &html_open();
    print "<HR>\n";
    print "<STRONG>Error:</STRONG> @_\n";
    print "<P>\n";
    print $text{'use_the_link'};
    print "</P>\n<HR>\n";
    &html_close();
}


# Get Aliases for user so that they can be recognized through .forward
sub get_aliases {

    $aliasename = ""; 
    if ($config{'aliase_path'} ne "") {
      open(ALIASEOUT, "grep -w $user $config{'aliase_path'}|") ||
	    &html_abort($text{'abort_cannot_read_aliases'});
        @aliaselist = <ALIASEOUT>;
        close(ALIASEOUT);

        $aindx = 0;

        while ($aindx < @aliaselist) {
            @name = split(/:/,$aliaselist[$aindx]);
            if ($name[0] =~ /^$config{'username_rexexp'}$/) { 
                $aliasename .= " $name[0]";
            }
            $aindx += 1;
        }
    }
}

# Get Virtual User for user so that they can be recognized through .forward
sub get_virtuser {

    if ($config{'virtuser_path'} ne "") {
      open(VIRTUSEROUT, "grep -w $user $config{'virtuser_path'}|") ||
            &html_abort($text{'abort_cannot_read_virtuser'});
        @virtuserlist = <VIRTUSEROUT>;
        close(VIRTUSEROUT);

        $vindx = 0;

        while ($vindx < @virtuserlist) {
            @name = split(/@/,$virtuserlist[$vindx]);
            if ($name[0] =~ /^$config{'username_rexexp'}$/) {
                $aliasename .= " $name[0]";
            }
            $vindx += 1;
        }
    }
}


# Creat a temporary file containing nothing
sub create_null_file {
    open(OUTPUT,">$tmp") || &html_abort($text{'abort_cannot_create_file'}.$tmp);
    close(OUTPUT);
}


# Create a temporary file containing the vacation message
sub create_vacation_msg {
    open(OUTPUT,">$tmp") || &html_abort($text{'abort_cannot_create_file'}.$tmp);
    $mess =~ s/\r//g;	print OUTPUT $mess;
    close(OUTPUT);
}


# Creat a temporary .forward file containing the call to vacation and maybe
# some mail addresses
sub create_forward {

    # Get Aliases for user so that they can be recognized through .forward
    # Aliase name can ONLY be in the format of: Firstname.Lastname
    &get_aliases;
    &get_virtuser;

    open(OUTPUT,">$tmp") ||
	&html_abort($text{'abort_cannot_create_file'}.$tmp);

    if ( $aliasename gt "") 
    {
      # vacation takes aliases in the following form
      #   vacation [-a alias1] [-a alias2]...
      $parm_aliasenames = $aliasename;
      $parm_aliasenames =~ s/ / -a /g;
    };

    # print the line for vacation
    if ( $act_vac eq "set" ) {
	if ( $hold eq "yes" ) {
	    print OUTPUT 
		"\\$user, \"|$config{'vacation_path'} $parm_aliasenames $user\"\n";
	}
	else {
	    print OUTPUT "\"|$config{'vacation_path'} $parm_aliasenames $user\"\n";
	}
    }

    # print lines for forwarding if activated and addresses are given
    if ( $act_fwd eq "set" &&
 	 defined($fwrd) && 
	 $fwrd =~ /./ && 
	 $fwrd ne "<$text{'index_forward_msg'}>" ) {
	print OUTPUT "$fwrd";
    }
	
    close(OUTPUT);
}


# check if the user given as argument does exist
# if so, return the information we got from get_users_list
# if not, do a html_abort()
sub check_if_user_exists {
    my $user = $_[0];
    
    if ( defined( $user ) ) {

	# use the useradmin module to get a list of users
	do "../user_chooser.cgi";
	@ulist = &get_users_list;

	# now check if the user is in this list
	foreach $u (@ulist) {
	    if ($u->[0] eq $user) {
		# ok, we found  it and can stop
		return $u;
	    }
        }
    }

    # if we reach this point, we didn't find the user
    &html_abort( $text{'abort_nouser'} . ": " . $user );
}


# check if the user logged in is allowed to
# change settings for the given user
sub authenticate {
    my %access = &get_module_acl();

    # first of all, check if the given user does exist.
    $userinfo = &check_if_user_exists( $user );

    if ( $access{'editing'} eq "all" ) {
	# the logged in user is allowed to change everything
	# but check if the given user does exist
	return;
    }
    elsif ( $access{'editing'} eq "this" ) {
	# the user is allowed to change just his personal settings
	if ( $user ne $ENV{'REMOTE_USER'} ) {
	    &html_abort( $text{'abort_noperm'} );
	}
	else {
	    return;
	}
    }
    elsif ( $access{'editing'} eq "listed" ) {
	# editing is allowed for users listed in $access{'listed'}
	my @allowedusers = split(/[, \t]+/,$access{'listed'});
	foreach $u (@allowedusers) {
	    if ( $user eq $u ) {
		# editing is allowed, stop now
		return;
	    }
	}
	&html_abort( $text{'abort_noperm'} );
    }
    elsif ( $access{'editing'} eq "except" ) {
	# editing is allowed for users all but the ones listed in $access{'except'}
	my @deniedusers = split(/[, \t]+/,$access{'except'});
	foreach $u (@deniedusers) {
	    if ( $user eq $u ) {
		# editing is not allowed, abort!
		&html_abort( $text{'abort_noperm'} );
	    }
	}
	return;
    }
    elsif ( $access{'editing'} eq "range" ) {
	# user is allowed to modify users whos uid is in a given range
	my $uid = $userinfo->[2];
	if ( $uid >= $access{'lowbound'} &&
	     $uid <= $access{'highbound'} ) {
	    # ok, uid is the given range
	    return;
	}
	&html_abort( $text{'abort_noperm'} );
    }	

    # the config file is not valid!
    &html_abort( $text{'abort_invalid_config'} . ": <em>editing</em>" );
}


# returns the name of the standard group for the given user
sub get_group {
    my $user=@_[0];

    # We use the system's groups commando, so we have no troubles with
    # NIS and so on.
    open(INPUT,"groups $user |");
    @group = <INPUT>;
    close(INPUT);

    $group = (split /: /, @group[0])[1];
    chomp($group);              # cut off the trailing new line
    # We want only the default (first) group.
    $group = (split / /,$group)[0];
    &html_abort( $text{'abort_nogroup'} ) unless ( defined($group) );

    return $group;
}


# returns the given user's home directory using finger
sub get_home {
    my $user=@_[0];
    my @list;
    my $home;

    @list=getpwnam($user);
    $home=@list[7];
    &html_abort( $text{'abort_nohome'} ) unless ( defined($home) );

    return $home;
}


# sets user, group and permissions for the given file
# params: <filename>, <user>, <group>[, <mode>] 
sub set_perms {
    my $file=@_[0];
    my $user=@_[1];
    my $group=@_[2];
    my $mode=@_[3];
    my $rv=0;

    $rv=system("/bin/chown $user:$group $file");
    &html_abort($text{'abort_owner_group'}.$file) unless ($rv == 0);

    if ( $mode ne "" ) {
	$rv=system("/bin/chmod 600 $file");
	&html_abort($text{'abort_perms'}.$file) unless ($rv == 0);
    }
}


sub unset_vacation_via_ftp {
    $ftp = Net::FTP->new($config{'server_domain'}, Debug => 0);
 
    unless ($ftp->login($user, $pass)) {
        $ftp->quit;
        &html_abort($text{'abort_login_failure'}."'<I>$user</I>'"); 
    }

    $ftp->delete(".forward");
    $ftp->delete(".vacation.msg");

    # Solaris uses .PAG and .DIR files.  While Linux only uses a .DB.
    # So we'll delete the appropriate files based on the configuration you entered

    if ($config{'os_type'} eq "solaris") {
        $ftp->delete(".vacation.pag");
        $ftp->delete(".vacation.dir");
    } else {
        $ftp->delete(".vacation.db");
    }

    $ftp->quit;
}


sub unset_vacation_local {

    # The authentication is only needed, if we are NOT in admin mode!
    &authenticate;

    # Get the users home directory
    my $home=&get_home($user);
    my @files=("$home/.vacation.msg", "$home/.forward");

    # Solaris uses .PAG and .DIR files.  While Linux only uses a .DB.
    # So we'll delete the appropriate files based on the configuration you entered
    if ($config{'os_type'} eq "solaris") {

	@files=(@files, "$home/.vacation.pag", "$home/.vacation.dir");
    }
    else {
        @files=(@files, "$home/.vacation.db");
    }
    
    # ok, now delete the files;
    if ( unlink(@files) ne ($#files+1) ) {
	my $filelist="";
	for (my $i=0; $i<=$#files; $i++) {
	    $filelist=$filelist."<li>".$files[$i]."\n";
	}
	&html_abort($text{'abort_remove_files'}."\n <ul>\n<pre>$filelist</pre>\n</ul>");
    }
}


# Here we use ftp to create the needed files
sub set_vacation_via_ftp {
    my $umask_cmd="SITE UMASK";
    my $ftp = Net::FTP->new($config{'server_domain'}, Debug => 0); 

    unless ($ftp->login($user, $pass)) {
        $ftp->quit; 
        &html_abort($text{'abort_login_failure'}."<I>$user</I>");
    }

    if ( $ftp->supported($umask_cmd) ) {
	# If the server supports the umask command, we set it to 0600
	$ftp->quot($umask_cmd,"066");
    }

    if ( $act_vac eq "set" ) {
	# Save and put .vacation.msg (w/o <cr>'s)
	&create_vacation_msg;
	$ftp->put($tmp, ".vacation.msg");


	# Solaris uses .PAG and .DIR files.  While Linux only uses a .DB.
	# and Slackware uses even a different vacation.db
	# FreeBSD has yet another DB format
	# So we'll FTP the appropriate files based on the configuration you entered
	if ($config{'os_type'} eq "solaris") {
	    
	    # Null .vacation.dir and .vacation.pag
	    &create_null_file;
	    $ftp->put($tmp, ".vacation.pag");
	    $ftp->put($tmp, ".vacation.dir");

	# This is for Slackware
		} elsif ($config{'os_type'} eq "slackware") {
		$ftp->put("vacation.db.slack", ".vacation.db");
		
	# This is for FreeBSD
		} elsif ($config{'os_type'} eq "freebsd") {
		$ftp->put("vacation.db.freebsd", ".vacation.db");

	# This is for Debian
		} elsif ($config{'os_type'} eq "debian") {
		$ftp->put("vacation.db.debian", ".vacation.db");

	} else {
		$ftp->put("vacation.db", ".vacation.db");
        }
    }

    # Save and put .forward file
    &create_forward;
    $ftp->put($tmp, ".forward");
    $ftp->quit;
}


# Here we can use normal file operations to generate the needed files
sub set_vacation_local {

    # The authentication is only needed, if we are NOT in admin mode!
    &authenticate;

    # Get the users home directory
    my $home=&get_home($user);
    my $group=&get_group($user);

    if ( $act_vac eq "set" ) {
	# Save the .vacation.msg file
	&create_vacation_msg;
	&copy($tmp, "$home/.vacation.msg");
	&set_perms("$home/.vacation.msg", $user, $group, "600");
	
	# Solaris uses .PAG and .DIR files.  While Linux only uses a .DB.
	# and Slackware uses even a different vacation.db 
	# So we'll copy the appropriate files based on the configuration you entered
	if ($config{'os_type'} eq "solaris") {
	    
	    # Null .vacation.dir and .vacation.pag
	    &create_null_file;
	    
	    &copy($tmp, "$home/.vacation.pag");
	    &set_perms("$home/.vacation.pag", $user, $group, "600");
	    &copy($tmp, "$home/.vacation.dir");
	    &set_perms("$home/.vacation.dir", $user, $group, "600");

	# This is for Slackware

	}
	elsif ($config{'os_type'} eq "slackware") {
            &copy("vacation.db.slack", "$home/.vacation.db");
            &set_perms("$home/.vacation.db", $user, $group, "600");
	}
	elsif ($config{'os_type'} eq "freebsd") {
            &copy("vacation.db.freebsd", "$home/.vacation.db");
            &set_perms("$home/.vacation.db", $user, $group, "600");
	}
	elsif ($config{'os_type'} eq "debian") {
            &copy("vacation.db.debian", "$home/.vacation.db");
            &set_perms("$home/.vacation.db", $user, $group, "600");
	}
	else {
	    &copy("vacation.db", "$home/.vacation.db");
	    &set_perms("$home/.vacation.db", $user, $group, "600");
	}
    }

    # Save and put .forward file
    &create_forward;
    &copy($tmp, "$home/.forward");
    &set_perms("$home/.forward", $user, $group, "600");
}
